﻿using System;
using System.Collections.Generic;
using System.Messaging;
using MessageQueueExample.Infrastructure;
using MessageQueueExample.Interfaces;
using MessageQueueExample.Models;

namespace MessageQueueExample
{
    class Program
    {
        private const string QueuePath = @".\Private$\Orders";

        static void Main(string[] args)
        {
            CreateQueue();
            
            using (var queue = new MessageQueue(QueuePath))
            {
                queue.Formatter = new BinaryMessageFormatter();

                CreateOrderMessages(queue, 2);

                BroadcastSequentialWorkers(queue, 3);

                //BroadcastParallelWorkers(queue, 3);
            }
            
            Console.ReadLine();
        }

        private static void CreateQueue()
        {
            if (!MessageQueue.Exists(@".\Private$\Orders"))
            {
                MessageQueue.Create(@".\Private$\Orders");
            } 
        }

        private static void CreateOrderMessages(MessageQueue queue, int count)
        {
            for (var orderId = 1; orderId <= count; orderId++)
            {
                var order = new OrderModel()
                {
                    Id = orderId,
                    Name = string.Format("Order {0}", orderId)
                };
                queue.Send(order);
            }
        }

        private static void BroadcastSequentialWorkers(MessageQueue queue, int count)
        {
            var workers = CreateCommands(count);
            var broadcastCommand = new CompositeSequentialBroadcastCommand<OrderModel>(workers);
            var queueMonitor = new QueueMonitor<OrderModel>(queue, broadcastCommand);
            queueMonitor.Start();
        }

        private static void BroadcastParallelWorkers(MessageQueue queue, int count)
        {
            var workers = CreateCommands(count);
            var broadcastCommand = new CompositeParallelBroadcastCommand<OrderModel>(workers);
            var queueMonitor = new QueueMonitor<OrderModel>(queue, broadcastCommand);
            queueMonitor.Start();
        }
        
        private static IEnumerable<ICommand<OrderModel>> CreateCommands(int count)
        {
            for (var workerId = 1; workerId <= count; workerId++)
            {
                yield return new OrderProcessorCommand(workerId);
            }
        } 
    }
}
